﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Sbo.Logex.Dotani.BusinessObjects;
using Produmex.Sbo.Logex.Dotani.Providers;
using Produmex.Sbo.Logex.Dotani.ViewObjects.Definitions.Views;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// Sub-flow : Select location of case item
    /// </summary>
    public class WorkflowScript_AdHocAfterItemToPickSelectedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		// Input parameters * do not change *
		public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

		public ReadOnlyBinder<PmxItemInfo> ItemInfo;
		public ReadOnlyBinder<int> PickListDocEntry;
		public ReadOnlyBinder<string> PmxWhsCode;
		public ReadOnlyBinder<DataRow> SelectedProductDataRow;


		// Output parameters * do not change *
		public ReadWriteBinder<bool> BackRequested;
		// Sub-flows
		// <none>


		/// <summary>
		/// Definitions
		/// </summary>
		private static class Definitions
		{
			/// <summary>
			/// The separator for separating strings values
			/// </summary>
			public const string SEPARATOR = ", ";
		}


		public WorkflowScript_AdHocAfterItemToPickSelectedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {

        }


        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {

            //Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            string initialErrorKey = null;
            Message msg;
			string query = null;
			PmxItemInfo itemInfo = ItemInfo.Get();
			string pmxWhsCode = PmxWhsCode.Get();
			DataRow selectedProductDataRow = SelectedProductDataRow.Get();

			DataSet dsLocations = null;
			string pickLocationCode = null;
			bool isSelectLocation = false;

			//Check if item is available on a pick location
			query = BuildQuery.GetStock(itemInfo.ItemCode, true, pmxWhsCode, sboProviderService.GetDbTool());
			DataSet ds = sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(ds) > 0)
			{
				//Nothing to do, because the stock is available on a pick location
				return;
			}


			Step_IdentifyLocation:

			#region IDENTIFY LOCATION

			pickLocationCode = null;
			isSelectLocation = false;


			string strListOfLocationsForItem = selectedProductDataRow[ProductDataSet.Columns.LOCATION_CODES.Name] as string;
			double stillToPick = Convert.ToDouble(selectedProductDataRow[ProductDataSet.Columns.STILL_TO_PICK.Name]);

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Picking.IIdentifyLocationAdHocPickingScreen),
			DefaultCultureInfo.Get(), BuildParamCollection(
				"InitialErrorKey", initialErrorKey,
				"TitleKey", "MSG_TITLE_SCAN_LOCATION",
				"AllowedButtons", new string[] { "MSG_BUTTON_SELECT_LOCATION" },
				"ItemCode", itemInfo.ItemCode,
				"ItemDescription", itemInfo.CustomItemDescription,
				"ItemPackagingTypes", null,
				"BatchNumber", Foundation.SlimScreen.WinGui.FormattingHelper.FormatQuantity(stillToPick, itemInfo.UomDecimals, itemInfo.InventoryUom, ""),
				"MessageKeyLabelBatchNumber", "MSG_LBL_STILL_TO_PICK",
				"BestBeforeDate", null,
				"LocationCodes", strListOfLocationsForItem,
				"ImagePath", itemInfo.PictureName
				));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				isSelectLocation = true;
				goto Step_SelectLocation;
			}

			if (msg.Name.EndsWith(".LocationEntered"))
			{
				pickLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

				goto Step_SelectLocation;
			}

			//Go back
			BackRequested.Set(true);
			return;

			#endregion

			Step_SelectLocation:

			#region SELECT LOCATION

			// Get locations
			query = BuildQuery.GetAllLocations(pmxWhsCode, pickLocationCode, itemInfo.ItemCode, true, sboProviderService.GetDbTool());
			dsLocations = sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsLocations) == 0)
			{
				// Inform the user that the case item is not located on that location
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
					"MessageKey", "The case item is not stored on this location, please scan another location.",
						"NoTranslationOfMessageKey", true,
						"ShowButton", true));
				WaitForMessage();

				//go back
				goto Step_IdentifyLocation;
			}

			if (this.GetNumberOfRows(dsLocations) == 1 &&
				!isSelectLocation)
			{

				//auto select
				pickLocationCode = dsLocations.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name] as string;

				goto StepAddLink;
			}


			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
				"InitialErrorKey", initialErrorKey,
				"TitleKey", "MSG_TITLE_SELECT_PICK_LOCATION",
				"LocationDS", dsLocations,
				"ShowStopButton", false));

			msg = WaitForMessage();
			// Reset latest error
			initialErrorKey = null;
			pickLocationCode = null;

			if (msg.Name.EndsWith(".LocationSelected"))
			{
				pickLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");
				goto StepAddLink;
			}


			//go back
			goto Step_IdentifyLocation;


			#endregion

		StepAddLink:
			#region ADD LINK

			try
			{
				sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, delegate (PmxDbConnection conn, object[] parameters)
				{
					//get the pick list line numbers for the current item.
					query = BuildQuery.GetPickListLineNums(PickListDocEntry.Get(), itemInfo.ItemCode, sboProviderService.GetDbTool());
					List<int> pickListLines = new List<int>();
					using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, conn))
					{
						while (!rs.EoF)
						{
							pickListLines.Add(rs.GetTypedValue<int>(PmxPickListLineTable.Columns.LineNum.NAME));
							rs.MoveNext();
						}
					}

					PmxLinkLocationToPickListDotaniProvider linkProv = new PmxLinkLocationToPickListDotaniProvider(conn);

					foreach (int lineNum in pickListLines)
					{
						using (PmxLinkLocationToPickListDotani link = linkProv.GetNewBO())
						{
							link.LocationCode = pickLocationCode;
							link.PickListDocEntry = PickListDocEntry.Get();
							link.PickListLineNum = lineNum;
							linkProv.AddBO(link);
						}
					}

					return null;

				});
			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();

				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}

				//retry?
				//ask user if he wants to retry
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_RETRY_UPDATE",
						"MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
				msg = WaitForMessage();

				if (msg.Name.ToLower().EndsWith(".yes"))
				{
					goto StepAddLink;
				}


				goto StepReturn;
			}


			goto StepReturn;
			#endregion



			StepReturn:

			BackRequested.Set(true);

		}
        #endregion


        #region HELPERS

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }



		#endregion


		#region BUILD QUERY

		/// <summary>
		/// Static class grouping our native queries
		/// </summary>
		private class BuildQuery
		{

			/// <summary>
			/// Gets the stock.
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <param name="pickLocation">The pick location.</param>
			/// <param name="pmxWhsCode">The PMX WHS code.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetStock(string itemCode, bool? pickLocation, string pmxWhsCode, DbTool dbTool)
			{
				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
				strBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME);


				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);


				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				strBuilder.Append(" AND ");
				strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pmxWhsCode, CultureInfo.InvariantCulture));


				if (pickLocation.HasValue)
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.IsPickLoc.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickLocation.Value? SboBooleanColumnValues.Y.VALUE : SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
				}

				strBuilder.Append(" ORDER BY ");
				strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME));


				return strBuilder.ToString();
			}


			/// <summary>
			/// Gets all locations.
			/// </summary>
			/// <param name="pmxWarehouseCode">The PMX warehouse code.</param>
			/// <param name="locationCode">The location code.</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="allowAdhocPickFromBulk">if set to <c>true</c> [allow adhoc pick from bulk].</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetAllLocations(string pmxWarehouseCode, string locationCode, string itemCode, bool allowAdhocPickFromBulk, DbTool dbTool)
			{
				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
				strBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

				strBuilder.Append("\r\n INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

				strBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxCaseWeighItemDotaniViewDefinition.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxCaseWeighItemDotaniViewDefinition.NAME, PmxCaseWeighItemDotaniViewDefinition.Columns.CaseItem.NAME);


				strBuilder.Append(" LEFT JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxUserTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.LockedBy.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxUserTable.NAME, PmxUserTable.Columns.InternalKey.NAME);

				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				if (!allowAdhocPickFromBulk)
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsPickLoc.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
				}
				if (!string.IsNullOrEmpty(locationCode))
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));

				}


				if (!string.IsNullOrEmpty(pmxWarehouseCode))
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pmxWarehouseCode, CultureInfo.InvariantCulture));
				}

				if (!string.IsNullOrEmpty(itemCode))
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxCaseWeighItemDotaniViewDefinition.NAME, PmxCaseWeighItemDotaniViewDefinition.Columns.WeighItem.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
				}

				strBuilder.Append(" ORDER BY ");
				strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME));

				return strBuilder.ToString();
			}

			/// <summary>
			/// Gets the pick list line nums.
			/// </summary>
			/// <param name="pickListDocEntry">The pick list doc entry.</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="quantityPerUom">The quantity per uom.</param>
			/// <param name="batchToPick">The batch to pick.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetPickListLineNums(int pickListDocEntry, string itemCode, DbTool dbTool)
			{
				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

				strBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineNum.NAME);

				strBuilder.AppendFormat(", {0} - {1} AS {2}",
										  SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.Quantity.NAME),
						  SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QtyPicked.NAME),
						  SqlCommandHelper.EscapeColumnName(PmxPickListLineTable.Columns.Quantity.NAME)
							  );


				strBuilder.AppendFormat(", {0} - " + strBuilder.IFNULL + "({1}, 0) AS {2}",
										  SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QuantityUom2.NAME),
						  SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QtyPickedUom2.NAME),
						  SqlCommandHelper.EscapeColumnName(PmxPickListLineTable.Columns.QuantityUom2.NAME)
							  );

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QuantityPerUom.NAME);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.Uom.NAME);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.Uom2.NAME);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LogUnitIdentKey.NAME);


				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME));

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME);
				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME);
				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseType.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(Data.BusinessObjects.Definitions.PmxObjectTypes.PmxPickListProposal, CultureInfo.InvariantCulture));

				strBuilder.Append(" LEFT JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.ItemTransactionalInfoKey.NAME);


				strBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.PickListLineStatus.NAME);
				strBuilder.Append(" IN (");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.NotReady.VALUE, CultureInfo.InvariantCulture));
				strBuilder.Append(" , ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.Ready.VALUE, CultureInfo.InvariantCulture));
				strBuilder.Append(" , ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.PartiallyReady.VALUE, CultureInfo.InvariantCulture));
				strBuilder.Append(" , ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.PartiallyPicked.VALUE, CultureInfo.InvariantCulture));
				strBuilder.Append(" )");

				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture));

				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.ItemCode.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				return strBuilder.ToString();
			}


		}

		#endregion


	}


}
